#define WIN32_LEAN_AND_MEAN	// less crud from windows header
#include <windows.h>

/* 
 OP2 ScrollFix 
(gotta love originality)

12/23/08 by ZigZagJoe, with extensive help from Hooman

 Fixes Outpost2 scrolling around when it is not the focused window.

USE:

<Outpost2.ini>
 [Game]
 ...
 LoadAddons = "... OP2ScrollFix"
 
 [OP2ScrollFix]
 Dll = "OP2ScrollFix.dll"
 Enabled = 1

This dll can also be loaded with the old method, 
as it uses DllMain instead of mod_init and the like.
never got those to work for me anyways.

 "Enabled" can be set to 2 in the ini to disable ALL mouse scrolling

*/

// Locations
int callToScrollingFuncLoc	= 0x407E78; // 00407E78 E8 83FCFFFF CALL <Outpost2.IWnd:Pane:Detail.HandleSc>;  IWnd:Pane:Detail.HandleScrolling()
int scrollingFuncLoc		= 0x407B00; // 00407B00 83EC 28     SUB ESP,28                               ;  Function: IWnd:Pane:Detail.HandleScrolling()

// Prototypes
int Init();
void ScrollCheckFunc();

// Funcs
int __stdcall DllMain(HINSTANCE hInst, DWORD dwReason, LPVOID) {
	
	if (dwReason == DLL_PROCESS_ATTACH) {	
		DisableThreadLibraryCalls(hInst);	// so DllMain is not called more times than needed
											// windows thinks every DLL should know about new threads being made...
		
		// put in our hax
		return Init();
	} // use dwReason == DLL_PROCESS_DETACH if cleanup is necessary (called when exiting)
	  // as this only gets unloaded when op2 is exiting, and the func will not be called anymore,
	  // there is not much of a point in cleaning up after ourselves

	return true; // return true (1) if successful, false (0) if failed.
}

// Edit the call to the scroll func to call our function instead
int Init() {

	// read int from outpost2.ini        Section        Field   Def   File Name
	int enabledState = GetPrivateProfileInt("OP2ScrollFix","Enabled",1,"./Outpost2.ini");

	switch (enabledState) {
	case 1:
	case 2: {	// enabled

		HANDLE proc = NULL;	// Handle to OP2 process
		DWORD oldProtect;	// Old page protection status

		proc = OpenProcess(PROCESS_ALL_ACCESS,true,GetCurrentProcessId());

		if (proc == NULL) // this should never happen, except maybe if not admin? 
			return false; // probably should test that!

		// Unprotect the memory page containing the location
		VirtualProtectEx(proc,(void *)callToScrollingFuncLoc,5,PAGE_EXECUTE_READWRITE,&oldProtect);

		if (enabledState == 1) {

			// Operand is one byte long - data section is 4 bytes
			*(int*)(callToScrollingFuncLoc + 1) = (int)(&ScrollCheckFunc) - (callToScrollingFuncLoc + 1 + 4);
			// data portion of the instruction  = address of new function -  call to the scrolling func + sz(oper [1] + data [4])
		} else {
			// NOP the call out completely
			for (int o = 0; o < 5; o++) // need to replace the databytes and the operand, 
										// as NOPs do not have data bytes and would crash (or cause bad things)
				*(unsigned char*)(callToScrollingFuncLoc + o) = 0x90;
				// operand (1 byte long)                      = NOP
		}

		// Re-protect the memory page containing the location (to be safe)
		VirtualProtectEx(proc,(void *)callToScrollingFuncLoc,5,oldProtect,&oldProtect);

	}
	break;

	default:	// do not do anything for any other value
		break;
	}

	return true;
}

// Check if OP2 is focused (and possibly scroll if so)
void __declspec(naked) ScrollCheckFunc() {

	// declared naked because we do our own stack management
	// we do not need to back up the other registers,
	// because we are not messing with them at all!
	// probably should not use this if using non-ASM code...!

	__asm {	/* inline ASM */

		PUSH ECX						// put the class reference (this) on the stack because GetFocus trashes it 
			// EAX, ECX, EDX are the registers recommended for use by windows programmers, so any data in them initially
			// may or may not be kept intact. EAX rarely so, as that is the register designated for returns.

		CALL DWORD PTR GetFocus			// get the currently focused HWND 
			// (Win32 API). result goes in EAX. ECX will be trashed by this function

		MOV ECX, DWORD PTR ds:5695660	// location of the base of the main IWnd class

		CMP EAX, DWORD PTR [ECX+8]		// compare the 2 HWNDs
			// currently-focused HWND (in EAX) to OP2's HWND (offset 0x8 in IWnd, stored in ECX)
			// sets the zero flag if equal

		POP ECX							// restore the class reference off the stack (into ECX)

		JNZ end							// jump if not zero to label 'end'
			// only jumps if the zero flag is not set (if focusHwnd != op2Hwnd) 

		CALL scrollingFuncLoc			// call the original scroll function 

end:
		RETN 0							// return - end of function (go back to OP2 code)
	}
}

